@@ -3,7 +3,7 @@ import typing
 import starlette.exceptions
 
 
-class ParseError(Exception):
+class DecodeError(Exception):
     """
     Raised by a Codec when `decode` fails due to malformed syntax.
     """
@@ -20,15 +20,15 @@ class NoReverseMatch(Exception):
     Raised by a Router when `reverse_url` is passed an invalid handler name.
     """
 
-    pass
+    ...
 
 
 class NoCodecAvailable(Exception):
-    pass
+    ...
 
 
 class ConfigurationError(Exception):
-    pass
+    ...
 
 
 class ComponentNotFound(ConfigurationError):
@@ -45,10 +45,27 @@ class ComponentNotFound(ConfigurationError):
         return msg
 
 
+class WebSocketException(Exception):
+    def __init__(self, close_code: int):
+        self.close_code = close_code
+
+
+class WebSocketConnectionException(Exception):
+    ...
+
+
 class HTTPException(starlette.exceptions.HTTPException):
-    pass
+    ...
 
 
 class ValidationError(HTTPException):
     def __init__(self, detail: typing.Union[str, typing.Dict[str, typing.List[str]]], status_code: int = 400):
         super().__init__(status_code=status_code, detail=detail)
+
+
+class InputValidationError(ValidationError):
+    ...
+
+
+class OutputValidationError(ValidationError):
+    ...
