@@ -1,65 +1,70 @@
 import asyncio
-import enum
 import inspect
 import typing
-from dataclasses import dataclass
 from functools import wraps
 
+import marshmallow
 import starlette.routing
-from starlette.responses import JSONResponse, Response
+from starlette.concurrency import run_in_threadpool
 from starlette.routing import Match, Mount
 from starlette.types import ASGIApp, ASGIInstance, Receive, Scope, Send
 
-import marshmallow
-from starlette_api import http
+from starlette_api import http, websockets
 from starlette_api.components import Component
+from starlette_api.responses import APIResponse
+from starlette_api.types import Field, FieldLocation, HTTPMethod, OptBool, OptFloat, OptInt, OptStr
 from starlette_api.validation import get_output_schema
 
 __all__ = ["Route", "WebSocketRoute", "Router"]
 
 
-class Location(enum.Enum):
-    query = enum.auto()
-    path = enum.auto()
-    body = enum.auto()
-
-
-@dataclass
-class Field:
-    name: str
-    location: Location
-    schema: typing.Union[marshmallow.fields.Field, marshmallow.Schema]
-    required: bool = False
-
-
 FieldsMap = typing.Dict[str, Field]
 MethodsMap = typing.Dict[str, FieldsMap]
 
+PATH_SCHEMA_MAPPING = {
+    inspect.Signature.empty: lambda *args, **kwargs: None,
+    int: marshmallow.fields.Integer,
+    float: marshmallow.fields.Number,
+    str: marshmallow.fields.String,
+    bool: marshmallow.fields.Boolean,
+    http.PathParam: marshmallow.fields.String,
+}
+
+QUERY_SCHEMA_MAPPING = {
+    inspect.Signature.empty: lambda *args, **kwargs: None,
+    int: marshmallow.fields.Integer,
+    float: marshmallow.fields.Number,
+    bool: marshmallow.fields.Boolean,
+    str: marshmallow.fields.String,
+    OptInt: marshmallow.fields.Integer,
+    OptFloat: marshmallow.fields.Number,
+    OptBool: marshmallow.fields.Boolean,
+    OptStr: marshmallow.fields.String,
+    http.QueryParam: marshmallow.fields.String,
+}
+
 
 class FieldsMixin:
-    def _get_fields(self, router: "Router") -> typing.Tuple[MethodsMap, MethodsMap, typing.Dict[str, Field]]:
+    def _get_fields(
+        self, router: "Router"
+    ) -> typing.Tuple[MethodsMap, MethodsMap, typing.Dict[str, Field], typing.Dict[str, typing.Any]]:
         query_fields: MethodsMap = {}
         path_fields: MethodsMap = {}
         body_field: typing.Dict[str, Field] = {}
+        output_field: typing.Dict[str, typing.Any] = {}
 
-        if hasattr(self, "methods"):
+        if hasattr(self, "methods") and self.methods is not None:
             if inspect.isclass(self.endpoint):  # HTTP endpoint
-                methods = (
-                    [(m, getattr(self.endpoint, m.lower() if m != "HEAD" else "get")) for m in self.methods]
-                    if self.methods
-                    else []
-                )
+                methods = [(m, getattr(self.endpoint, m.lower() if m != "HEAD" else "get")) for m in self.methods]
             else:  # HTTP function
                 methods = [(m, self.endpoint) for m in self.methods] if self.methods else []
         else:  # Websocket
             methods = [("GET", self.endpoint)]
 
-        for method, handler in methods:
-            query_fields[method], path_fields[method], body_field[method] = self._get_fields_from_handler(
-                handler, router
-            )
+        for m, h in methods:
+            query_fields[m], path_fields[m], body_field[m], output_field[m] = self._get_fields_from_handler(h, router)
 
-        return query_fields, path_fields, body_field
+        return query_fields, path_fields, body_field, output_field
 
     def _get_parameters_from_handler(
         self, handler: typing.Callable, router: "Router"
@@ -78,7 +83,7 @@ class FieldsMixin:
 
     def _get_fields_from_handler(
         self, handler: typing.Callable, router: "Router"
-    ) -> typing.Tuple[FieldsMap, FieldsMap, Field]:
+    ) -> typing.Tuple[FieldsMap, FieldsMap, Field, typing.Any]:
         query_fields: FieldsMap = {}
         path_fields: FieldsMap = {}
         body_field: Field = None
@@ -90,35 +95,38 @@ class FieldsMixin:
 
             # Matches as path param
             if name in self.param_convertors.keys():
-                schema = {
-                    param.empty: None,
-                    int: marshmallow.fields.Integer(required=True),
-                    float: marshmallow.fields.Number(required=True),
-                    str: marshmallow.fields.String(required=True),
-                }[param.annotation]
-                path_fields[name] = Field(name=name, location=Location.path, schema=schema)
+                try:
+                    schema = PATH_SCHEMA_MAPPING[param.annotation]
+                except KeyError:
+                    schema = marshmallow.fields.String
+
+                path_fields[name] = Field(
+                    name=name, location=FieldLocation.path, schema=schema(required=True), required=True
+                )
 
             # Matches as query param
-            elif param.annotation in (param.empty, int, float, bool, str, http.QueryParam):
-                if param.default is param.empty:
-                    kwargs = {"required": True}
+            elif param.annotation in QUERY_SCHEMA_MAPPING:
+                if param.annotation in (OptInt, OptFloat, OptBool, OptStr) or param.default is not param.empty:
+                    required = False
+                    kwargs = {"missing": param.default if param.default is not param.empty else None}
                 else:
-                    kwargs = {"missing": param.default}
-                schema = {
-                    param.empty: None,
-                    int: marshmallow.fields.Integer(**kwargs),
-                    float: marshmallow.fields.Number(**kwargs),
-                    bool: marshmallow.fields.Boolean(**kwargs),
-                    str: marshmallow.fields.String(**kwargs),
-                    http.QueryParam: marshmallow.fields.String(**kwargs),
-                }[param.annotation]
-                query_fields[name] = Field(name=name, location=Location.query, schema=schema)
+                    required = True
+                    kwargs = {"required": True}
+
+                query_fields[name] = Field(
+                    name=name,
+                    location=FieldLocation.query,
+                    schema=QUERY_SCHEMA_MAPPING[param.annotation](**kwargs),
+                    required=required,
+                )
 
             # Body params
             elif inspect.isclass(param.annotation) and issubclass(param.annotation, marshmallow.Schema):
-                body_field = Field(name=name, location=Location.body, schema=param.annotation())
+                body_field = Field(name=name, location=FieldLocation.body, schema=param.annotation())
 
-        return query_fields, path_fields, body_field
+        output_field = inspect.signature(handler).return_annotation
+
+        return query_fields, path_fields, body_field, output_field
 
 
 class Route(starlette.routing.Route, FieldsMixin):
@@ -126,10 +134,13 @@ class Route(starlette.routing.Route, FieldsMixin):
         super().__init__(path, endpoint=endpoint, **kwargs)
 
         # Replace function with another wrapper that uses the injector
-        if inspect.isfunction(endpoint):
+        if inspect.isfunction(endpoint) or inspect.ismethod(endpoint):
             self.app = self.endpoint_wrapper(endpoint)
 
-        self.query_fields, self.path_fields, self.body_field = self._get_fields(router)
+        if self.methods is None:
+            self.methods = [m for m in HTTPMethod.__members__.keys() if hasattr(self, m.lower())]
+
+        self.query_fields, self.path_fields, self.body_field, self.output_field = self._get_fields(router)
 
     def endpoint_wrapper(self, endpoint: typing.Callable) -> ASGIApp:
         """
@@ -151,6 +162,7 @@ class Route(starlette.routing.Route, FieldsMixin):
                     "app": app,
                     "path_params": route_scope["path_params"],
                     "route": route,
+                    "request": http.Request(scope, receive),
                 }
 
                 injected_func = await app.injector.inject(endpoint, state)
@@ -158,18 +170,15 @@ class Route(starlette.routing.Route, FieldsMixin):
                 if asyncio.iscoroutinefunction(endpoint):
                     response = await injected_func()
                 else:
-                    response = injected_func()
-
-                # Use output schema to validate and format data
-                output_schema = get_output_schema(endpoint)
-                if output_schema:
-                    response = output_schema.dump(response)
+                    response = await run_in_threadpool(injected_func)
 
                 # Wrap response data with a proper response class
                 if isinstance(response, (dict, list)):
-                    response = JSONResponse(response)
+                    response = APIResponse(content=response, schema=get_output_schema(endpoint))
                 elif isinstance(response, str):
-                    response = Response(response)
+                    response = APIResponse(content=response)
+                elif response is None:
+                    response = APIResponse(content="")
 
                 await response(receive, send)
 
@@ -186,7 +195,7 @@ class WebSocketRoute(starlette.routing.WebSocketRoute, FieldsMixin):
         if inspect.isfunction(endpoint):
             self.app = self.endpoint_wrapper(endpoint)
 
-        self.query_fields, self.path_fields, self.body_field = self._get_fields(router)
+        self.query_fields, self.path_fields, self.body_field, self.output_field = self._get_fields(router)
 
     def endpoint_wrapper(self, endpoint: typing.Callable) -> ASGIApp:
         """
@@ -208,6 +217,7 @@ class WebSocketRoute(starlette.routing.WebSocketRoute, FieldsMixin):
                     "app": app,
                     "path_params": route_scope["path_params"],
                     "route": route,
+                    "websocket": websockets.WebSocket(scope, receive, send),
                 }
 
                 injected_func = await app.injector.inject(endpoint, state)
@@ -223,6 +233,9 @@ class WebSocketRoute(starlette.routing.WebSocketRoute, FieldsMixin):
 class Router(starlette.routing.Router):
     def __init__(self, components: typing.Optional[typing.List[Component]] = None, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        if components is None:
+            components = []
+
         self.components = components
 
     def add_route(
@@ -244,6 +257,7 @@ class Router(starlette.routing.Router):
         if isinstance(app, Router):
             app.components = self.components
 
+        path = path.rstrip("/")
         route = Mount(path, app=app, name=name)
         self.routes.append(route)
 
@@ -264,20 +278,21 @@ class Router(starlette.routing.Router):
         return decorator
 
     def get_route_from_scope(self, scope) -> typing.Tuple[Route, typing.Optional[typing.Dict]]:
-        if "root_path" in scope:
-            scope["path"] = scope["root_path"] + scope["path"]
-            del scope["root_path"]
-
         partial = None
 
         for route in self.routes:
+            if isinstance(route, Mount):
+                path = scope.get("path", "")
+                root_path = scope.pop("root_path", "")
+                scope["path"] = root_path + path
+
             match, child_scope = route.matches(scope)
             if match == Match.FULL:
                 scope.update(child_scope)
 
                 if isinstance(route, Mount):
-                    del scope["root_path"]
-                    return route.app.get_route_from_scope(scope)
+                    route, mount_scope = route.app.get_route_from_scope(scope)
+                    return route, mount_scope
 
                 return route, scope
             elif match == Match.PARTIAL and partial is None:
